part of tecfy_core_client_package;

class TimeProperty extends StatefulWidget {
  final String? label;
  final String? description;
  final bool isRequired;
  final dynamic initalValue;
  final void Function(int?)? onChanged;
  TimeProperty({
    this.label,
    this.description,
    this.isRequired = false,
    this.initalValue,
    this.onChanged,
  });
  @override
  _TimePropertyState createState() => _TimePropertyState();
}

class _TimePropertyState extends State<TimeProperty> {
  int? _selectedValue;
  int _timeDiff = 0;

  @override
  void initState() {
    _timeDiff = intl.DateFormat('H:m')
        .parseUTC('12:0')
        .difference(intl.DateFormat('H:m').parse('12:0'))
        .inMinutes
        .toInt();
    print('initalValue : ${widget.initalValue} timeDiff : $_timeDiff');
    if (widget.initalValue != null) {
      if (widget.initalValue is String) {
        print('trying to parse');
        _selectedValue = int.parse(widget.initalValue) + _timeDiff;
      } else if (widget.initalValue is int) {
        _selectedValue = widget.initalValue + _timeDiff;
      }
    }
    super.initState();
  }

  void _onPressed() async {
    var value = await showTimePicker(
      builder: (context, child) {
        return Theme(
          data: Theme.of(context).copyWith(
            colorScheme: ColorScheme.light(
              primary: ThemeService.mainColor,
              onPrimary: Colors.white,
              onSurface: Colors.black,
            ),
            textButtonTheme: TextButtonThemeData(
              style: TextButton.styleFrom(
                primary: Colors.black,
              ),
            ),
          ),
          child: child ?? Container(),
        );
      },
      context: context,
      //firstDate: DateTime(1900),
      initialTime: _getTimeOfDay() ?? TimeOfDay.now(),
      //lastDate: DateTime(2500)
    );
    setState(() {
      if (value != null) _selectedValue = (value.minute + (value.hour * 60));
    });
    widget.onChanged
        ?.call(_selectedValue == null ? null : _selectedValue! - _timeDiff);
  }

  TimeOfDay? _getTimeOfDay() {
    return _selectedValue == null
        ? null
        : TimeOfDay(hour: _selectedValue! ~/ 60, minute: _selectedValue! % 60);
  }

  _onPrint() {
    if (_selectedValue == null && widget.initalValue != null) {
      if (widget.initalValue is String) {
        print('trying to parse');
        _selectedValue = int.parse(widget.initalValue) + _timeDiff;
      } else if (widget.initalValue is int) {
        _selectedValue = widget.initalValue + _timeDiff;
      }
    }
  }

  @override
  Widget build(BuildContext context) {
    _onPrint();
    return MaterialButton(
      padding: EdgeInsets.all(0),
      onPressed: _onPressed,
      child: widget.label == null
          ? Tooltip(
              message: widget.description ?? '',
              child: Text(_selectedValue != null
                  ? '${_getTimeOfDay()?.hour.toString().padLeft(2, '0')}:${_getTimeOfDay()?.minute.toString().padLeft(2, '0')}'
                  : '[Click To Select]'))
          : Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: <Widget>[
                SizedBox(height: 10),
                Tooltip(
                    message: widget.description ?? '',
                    child: Text(
                      '${widget.label ?? ''}${(widget.isRequired) ? '*' : ''}',
                      style: ThemeService.labelStyle,
                    )),
                Text(_selectedValue != null
                    ? '${_getTimeOfDay()?.hour.toString().padLeft(2, '0')}:${_getTimeOfDay()?.minute.toString().padLeft(2, '0')}'
                    : '[Click To Select]'),
                SizedBox(height: 10),
                Container(height: 1, color: ThemeService.textSubColor)
              ],
            ),
    );
  }
}
